"""Generate emojione data."""
import os
import json
current_dir = os.path.dirname(os.path.abspath(__file__))
LICENSE = """
MIT license.

Copyright (c) http://www.emojione.com
"""


def get_unicode_alt(value):
    """Get alternate Unicode form or return the original."""

    return value['unicode_alt']


def parse(repo, tag):
    """Save test files."""
    # Load emoji database
    with open(os.path.join(current_dir, 'tags', repo, repo, 'emoji.json')) as f:
        emojis = json.loads(f.read())

    emoji_db = {}
    shortnames = set()
    aliases = {}
    for v in emojis.values():
        shortnames.add(v['shortname'])
        emoji_db[v['shortname']] = {
            'name': v['name'],
            'unicode': v['unicode'],
            'category': v['category']
        }
        alt = get_unicode_alt(v)
        if alt:
            emoji_db[v['shortname']]['unicode_alt'] = alt

        for alias in v['aliases']:
            aliases[alias] = v['shortname']

    # Save test files
    for test in ('png', 'png sprite', 'svg', 'svg sprite', 'entities', 'long title', 'no title'):
        with open('../tests/extensions/emoji/emoji1 (%s).txt' % test, 'w') as f:
            f.write('# Emojis\n')
            count = 0
            for emoji in sorted(shortnames):
                f.write(''.join('{} {}<br>\n'.format(emoji[1:-1], emoji)))
                count += 1
                if test != 'png' and count == 10:
                    break

    # Write out essential info
    with open('../pymdownx/emoji1_db.py', 'w') as f:
        # Dump emoji db to file and strip out PY2 unicode specifiers
        f.write('"""Emojione autogen.\n\nGenerated from emojione source. Do not edit by hand.\n%s"""\n' % LICENSE)
        f.write('version = "%s"\n' % tag)
        f.write('name = "emojione"\n')
        f.write('emoji = %s\n' % json.dumps(emoji_db, sort_keys=True, indent=4, separators=(',', ': ')))
        f.write('aliases = %s\n' % json.dumps(aliases, sort_keys=True, indent=4, separators=(',', ': ')))



"""Generate emojione data."""
import os
import json
import sys
import copy
sys.path.append('../pymdownx/')
current_dir = os.path.dirname(os.path.abspath(__file__))

ADDITIONAL_ALIASES = {}

# Special emoji
SPECIAL_EMOJI = {
    # # An unofficial emoji supported by very few
    ":shibuya:": {
        "category": "travel",
        "name": "Shibuya 109",
        "unicode": "e50a"
    },

    # Skier skin tones (not supported by Unicode spec)
    ":skier_tone1:": {
        "category": "activity",
        "name": "skier: light skin tone",
        "unicode": "26f7-1f3fb"
    },
    ":skier_tone2:": {
        "category": "activity",
        "name": "skier: medium-light skin tone",
        "unicode": "26f7-1f3fc"
    },
    ":skier_tone3:": {
        "category": "activity",
        "name": "skier: medium skin tone",
        "unicode": "26f7-1f3fd"
    },
    ":skier_tone4:": {
        "category": "activity",
        "name": "skier: medium-dark skin tone",
        "unicode": "26f7-1f3fe"
    },
    ":skier_tone5:": {
        "category": "activity",
        "name": "skier: dark skin tone",
        "unicode": "26f7-1f3ff"
    },

    # Woman levitate (not supported in Unicode spec)
    ":woman_levitate:": {
        "category": "people",
        "name": "woman in business suit levitating",
        "unicode": "1f574-fe0f-200d-2640-fe0f",
        "aliases": ["woman_in_business_suit_levitating"]
    },
    ":woman_levitate_tone1:": {
        "category": "people",
        "name": "woman in business suit levitating: light skin tone",
        "unicode": "1f574-1f3fb-200d-2640-fe0f",
        "aliases": [
            "woman_in_business_suit_levitating_tone1",
            "woman_in_business_suit_levitating_light_skin_tone"
        ]
    },
    ":woman_leviate_tone2:": {
        "category": "people",
        "name": "woman in business suit levitating: medium-light skin tone",
        "unicode": "1f574-1f3fc-200d-2640-fe0f",
        "aliases": [
            "woman_in_business_suit_levitating_tone2",
            "woman_in_business_suit_levitating_medium_light_skin_tone"
        ]
    },
    ":woman_leviate_tone3:": {
        "category": "people",
        "name": "woman in business suit levitating: medium skin tone",
        "unicode": "1f574-1f3fd-200d-2640-fe0f",
        "aliases": [
            "woman_in_business_suit_levitating_tone3",
            "woman_in_business_suit_levitating_medium_skin_tone"
        ]
    },
    ":woman_leviate_tone4:": {
        "category": "people",
        "name": "woman in business suit levitating: medium-dark skin tone",
        "unicode": "1f574-1f3fe-200d-2640-fe0f",
        "aliases": [
            "woman_in_business_suit_levitating_tone4",
            "woman_in_business_suit_levitating_medium_dark_skin_tone"
        ]
    },
    ":woman_leviate_tone5:": {
        "category": "people",
        "name": "woman in business suit levitating: dark skin tone",
        "unicode": "1f574-1f3ff-200d-2640-fe0f",
        "aliases": [
            "woman_in_business_suit_levitating_tone5",
            "woman_in_business_suit_levitating_dark_skin_tone"
        ]
    },

    # Man in Santa hat and variants
    # Twitter has Santa Claus and man in Santa hat backwards
    ":man_in_santa_hat:": {
        "category": "people",
        "name": "man in santa hat",
        "unicode": "1f468-200d-1f384"
    },

    ":man_in_santa_hat_tone1:": {
        "category": "people",
        "name": "man in santa hat: light skin tone",
        "unicode": "1f468-1f3fb-200d-1f384"
    },

    ":man_in_santa_hat_tone2:": {
        "category": "people",
        "name": "man in santa hat: medium-light skin tone",
        "unicode": "1f468-1f3fc-200d-1f384"
    },

    ":man_in_santa_hat_tone3:": {
        "category": "people",
        "name": "man in santa hat: medium skin tone",
        "unicode": "1f468-1f3fd-200d-1f384"
    },

    ":man_in_santa_hat_tone4:": {
        "category": "people",
        "name": "man in santa hat: medium-dark skin tone",
        "unicode": "1f468-1f3fe-200d-1f384"
    },

    ":man_in_santa_hat_tone5:": {
        "category": "people",
        "name": "man in santa hat: dark skin tone",
        "unicode": "1f469-1f3ff-200d-1f384"
    },

    # Woman in Santa hat and variants
    # Twitter has Mrs. Claus and woman in Santa hat backwards
    ":woman_in_santa_hat:": {
        "category": "people",
        "name": "woman in santa hat",
        "unicode": "1f469-200d-1f384"
    },

    ":woman_in_santa_hat_tone1:": {
        "category": "people",
        "name": "woman in santa hat: light skin tone",
        "unicode": "1f469-1f3fb-200d-1f384"
    },

    ":woman_in_santa_hat_tone2:": {
        "category": "people",
        "name": "woman in santa hat: medium-light skin tone",
        "unicode": "1f468-1f3ff-200d-1f384"
    },

    ":woman_in_santa_hat_tone3:": {
        "category": "people",
        "name": "woman in santa hat: medium skin tone",
        "unicode": "1f469-1f3fe-200d-1f384"
    },

    ":woman_in_santa_hat_tone4:": {
        "category": "people",
        "name": "woman in santa hat: medium-dark skin tone",
        "unicode": "1f469-1f3fd-200d-1f384"
    },

    ":woman_in_santa_hat_tone5:": {
        "category": "people",
        "name": "woman in santa hat: dark skin tone",
        "unicode": "1f469-1f3fc-200d-1f384"
    }
}

IGNORE_EMOJI = [
    # Alternative man levitating
    # Per spec, "levitate" is already a man,
    # no need to explicitly have a man variant.
    "1f574-fe0f-200d-2642-fe0f",
    "1f574-1f3fb-200d-2642-fe0f",
    "1f574-1f3fc-200d-2642-fe0f",
    "1f574-1f3fd-200d-2642-fe0f",
    "1f574-1f3fe-200d-2642-fe0f",
    "1f574-1f3ff-200d-2642-fe0f",
]


def parse(repo, tag, jtag, emojis, emoji_aliases):
    """Save test files."""

    asset_path = os.path.join(current_dir, 'tags', repo, repo, 'assets', 'svg')

    emoji_db = {}
    shortnames = set()
    aliases = {}
    unsupported = []

    for asset in os.listdir(asset_path):
        if os.path.isfile(os.path.join(asset_path, asset)) and asset.endswith('.svg'):
            unicode_value = asset[:-4]
            unicode_alt = unicode_value
            codes = unicode_alt.split('-')
            diff = 4 - len(codes[0])
            if diff > 0:
                codes[0] = ('0' * diff) + codes[0]
                unicode_alt = '-'.join(codes)
            found = False
            for k, v in emojis.items():
                db_unicode = v.get('unicode_alt', v.get('unicode'))
                if (
                    db_unicode and
                    (unicode_alt == db_unicode or unicode_alt.replace('-fe0f', '') == db_unicode.replace('-fe0f', ''))
                ):
                    shortnames.add(k)
                    emoji_db[k] = copy.copy(v)
                    emoji_db[k]['unicode'] = unicode_value
                    if 'unicode_alt' in emoji_db[k] and unicode_alt == unicode_value:
                        del emoji_db[k]['unicode_alt']
                    elif unicode_alt != unicode_value:
                        emoji_db[k]['unicode_alt'] = unicode_alt
                    found = True
                    break
            if not found:
                unsupported.append(unicode_value)

    for k, v in emoji_aliases.items():
        if v in emoji_db:
            aliases[k] = v  # noqa: PERF403

    # Copy emoji only found in EmojiOne
    for k, v in SPECIAL_EMOJI.items():
        if k not in emoji_db and k not in aliases:
            shortnames.add(k)
            copy_emoji = copy.copy(v)
            if 'aliases' in copy_emoji:
                del copy_emoji['aliases']
            emoji_db[k] = copy_emoji
            code_point = v.get('unicode_alt', v['unicode'])
            if code_point in unsupported:
                index = unsupported.index(code_point)
                del unsupported[index]
            special_aliases = v.get('aliases', [])
            for spec_alias in special_aliases:
                if spec_alias not in aliases:
                    aliases[spec_alias] = k

    # Additional aliases
    for k, v in ADDITIONAL_ALIASES.items():
        if k in emoji_db:
            for a in v:
                if a not in aliases:
                    aliases[a] = k

    # Remove ignored emoji from unsupported list
    for ignore in IGNORE_EMOJI:
        if ignore in unsupported:
            index = unsupported.index(ignore)
            del unsupported[index]

    # Save test files
    for test in ('png', 'svg', 'entities'):
        with open('../tests/extensions/emoji/twemoji (%s).txt' % test, 'w') as f:
            f.write('# Emojis\n')
            count = 0
            for emoji in sorted(shortnames):
                f.write(''.join('{} {}<br>\n'.format(emoji[1:-1], emoji)))
                count += 1
                if test != 'png' and count == 10:
                    break

    # Write out essential info
    with open('../pymdownx/twemoji_db.py', 'w') as f:
        # Dump emoji db to file and strip out PY2 unicode specifiers
        f.write('"""Twemoji autogen.\n\nNames from emojione database. Do not edit by hand.\n"""\n')
        f.write('version = "%s"\n' % tag)
        f.write('index_version = "%s"\n' % jtag)
        f.write('name = "twemoji"\n')
        f.write('emoji = %s\n' % json.dumps(emoji_db, sort_keys=True, indent=4, separators=(',', ': ')))
        f.write('aliases = %s\n' % json.dumps(aliases, sort_keys=True, indent=4, separators=(',', ': ')))

    # Print unsupported emoji
    if unsupported:
        print('Unsupported:')
        for code in unsupported:
            print(code)



"""Generate gemoji data."""
import sys
import os
import json
current_dir = os.path.dirname(os.path.abspath(__file__))

U_JOIN = 0x200d
U_VARIATION_SELECTOR_16 = 0xfe0f
U_EXTRA = (U_JOIN, U_VARIATION_SELECTOR_16)

if sys.maxunicode == 0xFFFF:

    def get_code_points(s):
        """Get the Unicode code points."""

        pt = []

        def is_full_point(p, point):
            """
            Check if we have a full code point.

            Surrogates are stored in point.
            """
            v = ord(p)
            if 0xD800 <= v <= 0xDBFF:
                del point[:]
                point.append(p)
                return False
            if point and 0xDC00 <= v <= 0xDFFF:
                point.append(p)
                return True
            del point[:]
            return True

        return [(''.join(pt) if pt else c) for c in s if is_full_point(c, pt)]

    def get_ord(c):
        """Get Unicode ordinal number."""

        if len(c) == 2:
            high, low = (ord(p) for p in c)
            ordinal = (high - 0xD800) * 0x400 + low - 0xDC00 + 0x10000
        else:
            ordinal = ord(c)

        return ordinal

else:
    def get_code_points(s):
        """Get the Unicode code points."""

        return list(s)

    def get_ord(c):
        """Get Unicode ordinal number."""

        return ord(c)


def get_unicode(value):
    """Get Unicode."""

    uc = '-'.join(
        ['%04x' % get_ord(point) for point in get_code_points(value['emoji']) if get_ord(point) not in U_EXTRA]
    )

    uc_alt = '-'.join(
        ['%04x' % get_ord(point) for point in get_code_points(value['emoji'])]
    )

    if uc == uc_alt:
        uc_alt = None

    return uc, uc_alt


def get_gemoji_specific(value):
    """Get alternate Unicode form or return the original."""

    return value['aliases'][0]


def parse(repo, tag):
    """Save test files."""
    # Load emoji database
    with open(os.path.join(current_dir, 'tags', repo, repo, 'db', 'emoji.json'), 'r', encoding='utf-8') as f:
        emojis = json.loads(f.read())

    emoji_db = {}
    shortnames = set()
    aliases = {}
    for v in emojis:
        short = v['aliases'][0]
        shortnames.add(':%s:' % short)
        if 'emoji' in v:
            uc, uc_alt = get_unicode(v)
            emoji_db[':%s:' % short] = {
                'name': v.get('description', short),
                'unicode': uc,
                'category': v['category']
            }
            if uc_alt:
                emoji_db[':%s:' % short]['unicode_alt'] = uc_alt
        else:
            emoji_db[':%s:' % short] = {
                'name': v.get('description', short)
            }

        for alias in v['aliases'][1:]:
            aliases[':%s:' % alias] = ':%s:' % short

    # Save test files
    for test in ('png', 'entities'):
        with open('../tests/extensions/emoji/gemoji (%s).txt' % test, 'w') as f:
            f.write('# Emojis\n')
            count = 0
            for emoji in sorted(shortnames):
                f.write(''.join('{} {}<br>\n'.format(emoji[1:-1], emoji)))
                count += 1
                if test != 'png' and count == 10:
                    break

    with open(os.path.join(current_dir, 'tags', repo, repo, 'LICENSE')) as f:
        license_content = f.read()

    # Write out essential info
    with open('../pymdownx/gemoji_db.py', 'w') as f:
        # Dump emoji db to file and strip out PY2 unicode specifiers
        f.write('"""Gemoji autogen.\n\nGenerated from gemoji source. Do not edit by hand.\n\n%s"""\n' % license_content)
        f.write('version = "%s"\n' % tag)
        f.write('name = "gemoji"\n')
        f.write('emoji = %s\n' % json.dumps(emoji_db, sort_keys=True, indent=4, separators=(',', ': ')))
        f.write('aliases = %s\n' % json.dumps(aliases, sort_keys=True, indent=4, separators=(',', ': ')))



"""Generate gemoji index."""
import os
import requests
import json
import argparse
import shutil
import zipfile
import gen_emoji1
import gen_gemoji
import gen_twemoji
import gen_joypixels

current_dir = os.path.dirname(os.path.abspath(__file__))

GITHUB_API_HEADER = 'application/vnd.github.v3+json'
GITHUB_API = 'https://api.github.com'
GEMOJI = 'github/gemoji'
EMOJIONE = 'joypixels/emojione'
JOYPIXELS = 'joypixels/emoji-toolkit'
TWEMOJI = 'jdecked/twemoji'


def url_join(*args):
    """Join URL parts."""
    return '/'.join(args)


def get_github_emoji():
    """Get GitHub's usable emoji."""
    try:
        resp = requests.get(
            url_join(GITHUB_API, 'emojis'),
            headers={'Accept': GITHUB_API_HEADER},
            timeout=30
        )
    except Exception:
        return None

    return json.loads(resp.text)


def extract_tag(repo, file_location):
    """Extract tag from zip."""
    with zipfile.ZipFile(file_location, "r") as z:
        z.extractall(os.path.dirname(file_location))
    repo_dir = None
    base = os.path.dirname(file_location)
    for x in os.listdir(base):
        fullname = os.path.join(base, x)
        if os.path.isdir(fullname) and x.startswith(repo.replace('/', '-')):
            repo_dir = fullname
            break
    if repo_dir:
        os.rename(repo_dir, os.path.join(base, repo.replace('/', '-')))


def download_tag(repo, tag, url):
    """Download tag."""
    destination = os.path.join(current_dir, 'tags', repo.replace('/', '-'))
    if not os.path.exists(destination):
        os.makedirs(destination)
    else:
        print('Removing old tag...')
        shutil.rmtree(destination)
        os.makedirs(destination)
    file_location = os.path.join(destination, os.path.basename(url) + '.zip')
    print('Downloading: {} --> {}'.format(url, file_location))
    resp = requests.get(
        url,
        headers={'Accept': GITHUB_API_HEADER},
        stream=True
    )
    with open(file_location, 'wb') as f:
        for chunk in resp.iter_content(chunk_size=1024):
            if chunk:
                f.write(chunk)

    extract_tag(repo, file_location)


def select_tag(repo, no_download):
    """Get GitHub's usable emoji."""
    resp = requests.get(
        url_join(GITHUB_API, 'repos', repo, 'releases'),
        headers={'Accept': GITHUB_API_HEADER},
        timeout=50
    )

    assert resp.status_code == 200, "API call failed to get tag list!"
    tags = json.loads(resp.text)

    print('Select %s tag to use:' % repo)
    num_tags = len(tags)
    text = []
    for index in range(num_tags):
        text.append('    [%d] %s' % (index, tags[index]['tag_name']))
        if (index + 1) % 4 == 0:
            text.append('\n')
    if len(text) == 0 or text[-1] != '\n':
        text.append('\n')
    print(''.join(text))
    user_input = None
    while user_input is None:
        try:
            user_input = int(input('Select Tag > '))
        except Exception:
            user_input = None
        if user_input is not None and (user_input < 0 or user_input >= num_tags):
            user_input = None

    if not no_download:
        download_tag(repo, tags[user_input]['tag_name'], tags[user_input]['zipball_url'])
    return tags[user_input]['tag_name']


if __name__ == "__main__":
    parser = argparse.ArgumentParser(prog='gen_emoji', description='Generate emoji db files.')
    # Flag arguments
    parser.add_argument('--tag', default=None, help="Tag to use.")
    parser.add_argument('--joypixels-tag', default=None, help="Joypixels tag to use for Twemoji.")
    parser.add_argument('--joypixels-no-download', action='store_true', default=False, help="Skip Joypixels download.")
    parser.add_argument('--gemoji', action='store_true', default=False, help="Get Gemoji.")
    parser.add_argument('--emojione', action='store_true', default=False, help="Get Emojione.")
    parser.add_argument('--twemoji', action='store_true', default=False, help="Get Twemoji.")
    parser.add_argument('--no-download', action='store_true', default=False, help="Skip download and use local.")
    args = parser.parse_args()
    os.chdir(current_dir)
    if args.gemoji:
        if args.tag is None:
            tag = select_tag(GEMOJI, args.no_download)
        else:
            tag = args.tag
        gen_gemoji.parse(GEMOJI.replace('/', '-'), tag)
    if args.emojione:
        if args.tag is None:
            tag = select_tag(EMOJIONE, args.no_download)
        else:
            tag = args.tag
        gen_emoji1.parse(EMOJIONE.replace('/', '-'), tag)
    if args.twemoji:
        if args.joypixels_tag is None:
            jtag = select_tag(JOYPIXELS, args.joypixels_no_download)
        else:
            jtag = args.joypixels_tag
        db, aliases = gen_joypixels.parse(JOYPIXELS.replace('/', '-'), jtag)
        if args.tag is None:
            tag = select_tag(TWEMOJI, args.no_download)
        else:
            tag = args.tag
        gen_twemoji.parse(TWEMOJI.replace('/', '-'), tag, jtag, db, aliases)



"""Generate emojione data."""
import os
import json
current_dir = os.path.dirname(os.path.abspath(__file__))
LICENSE = """
MIT license.

Copyright (c) http://www.emojione.com
"""


def get_unicode_alt(value):
    """Get alternate Unicode form or return the original."""

    return value['code_points']['fully_qualified']


def parse(repo, tag):
    """Save test files."""
    # Load emoji database
    with open(os.path.join(current_dir, 'tags', repo, repo, 'emoji.json')) as f:
        emojis = json.loads(f.read())

    emoji_db = {}
    shortnames = set()
    aliases = {}
    for v in emojis.values():
        shortnames.add(v['shortname'])
        emoji_db[v['shortname']] = {
            'name': v['name'],
            'unicode': v['code_points']['base'],
            'category': v['category']
        }
        alt = get_unicode_alt(v)
        if alt and alt != v['code_points']['base']:
            emoji_db[v['shortname']]['unicode_alt'] = alt

        for alias in v['shortname_alternates']:
            aliases[alias] = v['shortname']

    return emoji_db, aliases
